#!/usr/bin/perl -w
#
# extraction des temperature/salinite CTD de 0 a 5m et TSG
#
# J Grelet IRD fevrier 2004 - sept 2008
# 
# $Id$ 

use strict; # necessite de declarer toutes les variables globales
#use diagnostics;

use Time::Local;
use Date::Manip;
use File::Basename;
use Data::Dumper;
use Getopt::Long;
use Switch;
use seawater;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION = '1.0';
our $AUTHOR  = 'jgrelet IRD sep 2008';
my  $DEBUG   = 0;
my  $ECHO;

# a mettre sous forme d'argument ligne de commande
my  $CONTEXTE = "AMMA";
my  $TIMEZONE = "GMT";
my  $FORMAT_DATE = "DMY";
my  $CODE = "1A";
my  $date_debut = "02/09/2008";
my  $date_fin   = "05/10/2008";
my  $tsgFile = "../THERMO/pirata-fr18.tsg";

my  $version;
my  $help;
my  $xml   = undef;
my  $ascii = 1;

my $code     = -1;  # code pour l'entete

my $hdr_file;
my $hdr_file_dec;
my $institute;
my $ctdType;
my $inst_sn;
my $pi;

my $cycle_mesure = "PIRATA-FR18";
my $ship         = "ANTEA";
my($seasave_version) = undef;
my($station, $depth,$T0,$T1,$flc,$ox0,$ox1,$nbin,$S0,$S1,$sigmateta,$sndvel,$trans, $TempCTD, $SalCTD);
my($year,$lat, $lat_dec, $lat_deg, $lat_min, $lat_hemi);
my($long, $long_dec, $long_deg, $long_min, $long_hemi);
my($julien, $h_date, $date, $mois, $jour, $annee, $heure, $min, $time);
my($dateTSG,$heureTSG,$SSJT,$SalTSG,$TempTSG,$flag);

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: ctd-tsg-fr18.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --cycle_mesure=<name>  cycle_mesure name\n";
  print STDERR   "    --ship=<ship_name>     ship name\n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "example:\n\$ perl ctd-tsg-fr18.pl data/fr18???.hdr --echo\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("cycle_mesure=s"  => \$cycle_mesure,    
               "ship=s"          => \$ship,  
               "echo"            => \$ECHO,  	       
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;
  $xml = undef if $ascii;    
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m) = @_;
  my $tmp = (($h * 60) + $m ) / 1440;
  return( $jj + $tmp ); 
}

#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
&usage if( $#ARGV == -1);
&get_options;

open( TSG_FILE, $tsgFile ) or warn("\n" . $tsgFile . ": " . $!);

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
open( DATA_FILE, $ARGV[0] );
while( <DATA_FILE> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $ship ) {
      ($ship) = $1;
       chop $ship;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($cycle_mesure) = $1 if ( not defined $cycle_mesure );
  }
  if( /Sea-Bird\s+(\w+\s+\d+)/ ) {
    ($ctdType) = $1  if ( not defined $ctdType );
  }
  if( /Software Version Seasave Win32 V\s+(\d+\.\d+)/ ) {
    ($seasave_version) = $1  if ( not defined $seasave_version );
  }
}  

# ouverture des fichiers de sortie, on met en minuscule, lower case
my $fileName =  $ARGV[0];
my ($name,$dir) =  fileparse $fileName;
$hdr_file = lc  $cycle_mesure.'-CTD-TSG.txt';
$hdr_file_dec = lc  $cycle_mesure.'-CTD-TSG_txt';

# ouverture des fichiers resultants, on sort si erreur
open( HDR_FILE, "+> $hdr_file" ) or die "Can't open file : $hdr_file\n";
open( HDR_FILE_DEC, "+> $hdr_file_dec" ) or die "Can't open file : $hdr_file_dec\n";

# ecriture des entetes
print HDR_FILE "$cycle_mesure  $ship\n";
print HDR_FILE "St  Heure   Date    Latitude   Longitude  TempCTD  TempTSG  SalCTD  SalTSG\n\n";
print HDR_FILE_DEC "$cycle_mesure  $ship\n";
print HDR_FILE_DEC "St  Anne Jour_julien   Latitude   Longitude  TempCTD  TempTSG  SalCTD  SalTSG\n\n";
close DATA_FILE;

# parcourt des fichiers .HDR et .asc
for( my $i = 0; $i <= $#ARGV; $i++ ){
  my $fileName = $ARGV[$i];
  open( DATA_FILE, $fileName ) or warn("\n" . $fileName . ": " . $!);
  print STDERR  "Lit: $fileName" if defined $ECHO;
  # on lit les fichiers d'extention .hdr contenant les entetes
  while( <DATA_FILE> ){           
    # les numeros de stations n'ont pas ete entres correctement dans les entetes
    # on utilise le numero du fichier correspondant
    if( $fileName =~ /.+fr18(\d{3})/ ) {
      $station = $1;
    }  
    #if( /Station\s*:\s*(\d+)/ )	{
    #  ($station)  = $1; 
    #}
    if( /System UpLoad Time =\s+(\.*)/)	{ # a modifier suivant le contexte
    ($time)  = /System UpLoad Time =\s+(\w+\s+\d+\s+\d+\s+\d+:\d+:\d+)/;	
    $date=&ParseDate($time);
    # transforme le day_of_year en julian day
    $julien = &UnixDate($time,"%j") -1;
    $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M") );
    $year   = &UnixDate($time,"%Y");	
    $h_date = &UnixDate($date,"%H:%M %d/%m/%Y");
    }
   
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_dec = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_dec = &position($long_deg, $long_min, $long_hemi);
    }
  }

  $SSJT = $SalTSG = $TempTSG = 0;
  &Date_Init("DateFormat=non-US", "TZ=UTC");
  while( <TSG_FILE> ) {
    (undef,$dateTSG,$heureTSG,undef,undef,undef,undef,$SSJT,$SalTSG,undef,$TempTSG) = split;
    $dateTSG = &ParseDate($dateTSG . " " . $heureTSG);
    #printf STDERR "%s %s\n", &UnixDate($dateTSG,"%d/%m/%Y %H:%M:%S"), 
    #                         &UnixDate($date,"%d/%m/%Y %H:%M:%S");
    $flag = &Date_Cmp($dateTSG, $date);
    if ($flag == 0) {
	    #print "\n$_\n";
      last;  # last is like break in C
    } elsif ($flag > 0) { 
	    #print "\n$_\n";
      last;
    } else { 
      $SSJT = $SalTSG = $TempTSG = 0;
    }
  }
  &Date_Init("DateFormat=US", "TZ=UTC");
  # affiche l'entete station a l'ecran	 
  printf STDERR "   %03d %s %02d%05.2f %s %03d%05.2f %s",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi if defined $ECHO;
  close DATA_FILE;

  # on lit les fichiers d'extension .asc contenant les donnees
  # attention aux majuscules/minuscules !!!!!
  $fileName =~ s/\.HDR/\.asc/i;
  # si fichier ouvert, on lit, sinon, on affiche une erreur sur STDERR
  if( open( DATA_FILE, $fileName ) ) {
    while( <DATA_FILE> ){           
      # on expurge toutes les lignes des entetes ainsi que les lignes vides
      # n'est pas necessaire avec asciiout
      if( not ( /^[*#]/ || /^\s*$/) ) {
        ($depth,$T0,$T1,$ox0,$ox1,$nbin,$S0,$S1,$sigmateta,$sndvel) = split;
        next if( $depth eq 'PrDM' ); 
        #$T68 = $T0 * 1.00024; 
        #$sndvel=&sw_svel($S0,$T68,$depth);
	
	      if ($depth == 4) {
	        $TempCTD = $T0;
          $SalCTD  = $S0;
        }  
        # ecriture dans la fichier 
      }
    }
    printf HDR_FILE  "%03d %s %02d%05.2f %s %03d%05.2f %s %6.3f  %6.3f  %6.3f  %6.3f\n",
      $station, $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
      $long_hemi, $TempCTD, $TempTSG, $SalCTD, $SalTSG ;
    printf HDR_FILE_DEC  "%03d %4d %10.9g %+08.4f %+09.4f %6.3f  %6.3f  %6.3f  %6.3f\n",
      $station, $year,$julien,$lat_dec,$long_dec, $TempCTD, $TempTSG, $SalCTD, $SalTSG ;
    printf STDERR " %6.3f  %6.3f  %6.3f  %6.3f\n",  $TempCTD, $TempTSG, $SalCTD, $SalTSG;
  } else {
    warn("\nErr: " . $fileName . ": " . $!);
  }
  $depth = 1e36;	  
  $. = 0; # remet le compteur de ligne a zero
  close DATA_FILE;
}

# return result for matlab

close HDR_FILE;
close HDR_FILE_DEC;

